<?php

namespace Drupal\translators_content\Access;

use Drupal\content_translation\Access\ContentTranslationManageAccessCheck;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\EntityTypeManager;
use Drupal\node\Entity\Node;
use Drupal\Core\Language\LanguageInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\translators\Services\TranslatorSkills;
use Drupal\translators_content\Handler\TranslatorsContentTranslationHandlerTrait;
use Symfony\Component\Routing\Route;

/**
 * Class TranslatorsContentManageAccess.
 *
 * @package Drupal\translators_content\Access
 */
class TranslatorsContentManageAccess extends ContentTranslationManageAccessCheck {
  use TranslatorsContentTranslationHandlerTrait;

  /**
   * Current user account.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;
  /**
   * Translator skills service.
   *
   * @var \Drupal\translators\Services\TranslatorSkills
   */
  protected $translatorSkills;

  /**
   * {@inheritdoc}
   */
  public function __construct(
    EntityTypeManager $entity_type_manager,
    LanguageManagerInterface $language_manager,
    AccountInterface $currentUser,
    TranslatorSkills $translatorSkills
  ) {
    parent::__construct($entity_type_manager, $language_manager);
    $this->currentUser      = $currentUser;
    $this->translatorSkills = $translatorSkills;
  }

  /**
   * {@inheritdoc}
   */
  public function access(
    Route $route,
    RouteMatchInterface $route_match,
    AccountInterface $currentUser,
    $source = NULL,
    $target = NULL,
    $language = NULL,
    $entity_type_id = NULL
  ) {
    /* @var \Drupal\Core\Entity\ContentEntityInterface $entity */
    if ($entity = $route_match->getParameter($entity_type_id)) {
      if ($route_match->getRouteName() === "entity.$entity_type_id.content_translation_overview") {
        return $this->checkTranslationOverviewAccess($entity);
      }
      $operation = $route->getRequirement('_access_content_translation_manage');
      $language = $this->languageManager->getLanguage($language) ?: $this->languageManager->getCurrentLanguage(LanguageInterface::TYPE_CONTENT);
      $entity_type = $this->entityTypeManager->getDefinition($entity_type_id);

      if (in_array($operation, ['update', 'delete'])) {
        // Translation operations cannot be performed on the default
        // translation.
        if ($language->getId() == $entity->getUntranslated()->language()->getId()) {
          return AccessResult::forbidden()->addCacheableDependency($entity);
        }
        // Editors have no access to the translation operations, as entity
        // access already grants them an equal or greater access level.
        $templates = ['update' => 'edit-form', 'delete' => 'delete-form'];
        if ($entity->access($operation) && $entity_type->hasLinkTemplate($templates[$operation])) {
          return AccessResult::forbidden()->cachePerPermissions();
        }
      }

      switch ($operation) {
        case 'create':
          $translations = $entity->getTranslationLanguages();
          $languages = $this->languageManager->getLanguages();
          $source_language = $this->languageManager->getLanguage($source) ?: $entity->language();
          $target_language = $this->languageManager->getLanguage($target) ?: $this->languageManager->getCurrentLanguage(LanguageInterface::TYPE_CONTENT);
          $is_new_translation = ($source_language->getId() != $target_language->getId()
            && isset($languages[$source_language->getId()])
            && isset($languages[$target_language->getId()])
            && !isset($translations[$target_language->getId()]));
          return AccessResult::allowedIf($is_new_translation)
            ->andIf($this->getTranslationAccess($entity, 'create', $source, $target));

        case 'delete':
          // @todo Remove this in https://www.drupal.org/node/2945956.
          /** @var \Drupal\Core\Access\AccessResultInterface $delete_access */
          $delete_access = \Drupal::service('content_translation.delete_access')->checkAccess($entity);
          $access = $this->checkAccess($entity, $language, $operation);
          return $delete_access->andIf($access);

        case 'update':
          return $this->checkAccess($entity, $language, $operation);
      }
    }

    // No opinion.
    return AccessResult::neutral();
  }

  /**
   * Access checking for translation overview page.
   *
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *   The entity being checked.
   *
   * @return \Drupal\Core\Access\AccessResultInterface
   *   An access result object.
   */
  public function checkTranslationOverviewAccess(ContentEntityInterface $entity) {
    if ($entity->isTranslatable()) {
      if (($entity instanceof Node)
        && ($this->currentUser->hasPermission('bypass node access'))) {
        return AccessResult::allowed()->cachePerPermissions()->addCacheableDependency($entity);
      }
      if ($this->getBundleTranslationAccess($entity)->isAllowed()
        && $this->hasOperationTranslationAccess()) {
        return AccessResult::allowed()->cachePerPermissions()->addCacheableDependency($entity);
      }
    }
    // No opinion.
    return AccessResult::neutral();
  }

  /**
   * {@inheritdoc}
   */
  public function checkAccess(ContentEntityInterface $entity, LanguageInterface $language = NULL, $operation) {
    $langcode = $language != NULL ? $language->getId() : $entity->language()->getId();
    $translations = $entity->getTranslationLanguages();
    $languages = $this->languageManager->getLanguages();
    $has_translation = isset($languages[$langcode])
      && isset($translations[$langcode]);
    $original_langcode = $entity->getUntranslated()->language()->getId();
    if ($langcode == $original_langcode) {
      return AccessResult::allowedIf($has_translation)
        ->andIf($this->getEntityAccess($operation, $entity));
    }
    return AccessResult::allowedIf($has_translation)
      ->andIf($this->getTranslationAccess($entity, $operation, $langcode));
  }

  /**
   * Performs entity create access check.
   *
   * @param string $entity_type_id
   *   Entity type machine name.
   * @param string $entity_bundle
   *   Entity bundle machine name.
   *
   * @return \Drupal\Core\Access\AccessResultInterface
   *   An access result object.
   */
  public function getEntityCreateAccess($entity_type_id, $entity_bundle) {
    if (($entity_type_id === 'node')
      && ($this->currentUser->hasPermission('bypass node access'))) {
      return AccessResult::allowed()
        ->cachePerPermissions();
    }
    elseif ($this->currentUser->hasPermission("create {$entity_bundle} content")) {
      return AccessResult::allowed()->cachePerPermissions();
    }
    elseif ($this->currentUser->hasPermission("translators_content create {$entity_bundle} content")) {
      $langcode = $this->languageManager->getCurrentLanguage(LanguageInterface::TYPE_CONTENT)->getId();
      return AccessResult::allowedIf($this->translatorSkills->hasLangcode($langcode))
        ->cachePerPermissions();
    }
    return AccessResult::neutral();
  }

  /**
   * Performs entity access check.
   *
   * @param string $operation
   *   Operation machine name.
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *   The entity being checked.
   *
   * @return \Drupal\Core\Access\AccessResultInterface
   *   An access result object.
   */
  public function getEntityAccess($operation, ContentEntityInterface $entity) {
    if (($entity instanceof Node)
      && ($this->currentUser->hasPermission('bypass node access'))) {
      return AccessResult::allowed()
        ->cachePerPermissions()->addCacheableDependency($entity);
    }
    $operation = $operation == 'update' ? 'edit' : $operation;
    if ($this->currentUser->hasPermission("$operation any {$entity->bundle()} content")) {
      return AccessResult::allowed()->cachePerPermissions();
    }
    elseif ($this->currentUser->hasPermission("$operation own {$entity->bundle()} content")
      && $this->currentUser->id() === $entity->getOwnerId()) {
      return AccessResult::allowed()
        ->cachePerPermissions()->addCacheableDependency($entity);
    }
    elseif ($this->currentUser->hasPermission("translators_content $operation any {$entity->bundle()} content")) {
      return AccessResult::allowedIf(
        $this->translatorSkills->hasLangcode($entity->language()->getId()))
        ->cachePerPermissions()->addCacheableDependency($entity);
    }
    return AccessResult::neutral();
  }

}
