<?php

namespace Drupal\Tests\translators_content\Functional;

use Drupal\node\Entity\Node;
use Drupal\Tests\BrowserTestBase;
use Drupal\Core\Language\Language;

/**
 * Class TranslatorsContentEntityPermissionsTest.
 *
 * @package Drupal\Tests\translators_content\Functional
 *
 * @group translators_content
 */
class TranslatorsContentEntityPermissionsTest extends BrowserTestBase {
  use TranslatorsContentTestsTrait;

  /**
   * {@inheritdoc}
   */
  public $profile = 'standard';
  /**
   * {@inheritdoc}
   */
  public static $modules = ['translators_content'];
  /**
   * Default core's langcodes 'und' and 'zxx', which we have to ignore.
   *
   * @var array
   */
  protected static $ignoredLangcodes = [
    Language::LANGCODE_NOT_SPECIFIED,
    Language::LANGCODE_NOT_APPLICABLE,
  ];

  /**
   * {@inheritdoc}
   */
  public function setUp() {
    parent::setUp();
    // Make all required configurations before testing.
    $this->setUpTest();
  }

  /**
   * Test "Create new content" permission.
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function testCreatePermission() {
    $creator = $this->createUser(['translators_content create article content']);
    $this->drupalLogin($creator);
    $this->registerTestSkills();

    foreach (static::$registeredSkills as $skill) {
      if ($skill === $this->defaultLanguage) {
        $this->drupalGet('node/add/article');
      }
      else {
        $this->drupalGet($skill . '/node/add/article');
      }
      $this->assertResponseCode(200);
    }

    foreach (static::$unregisteredSkills as $skill) {
      $this->drupalGet($skill . '/node/add/article');
      $this->assertResponseCode(403);
      $this->assertTextHelper('Access denied', FALSE);
    }
  }

  /**
   * Test "Edit any content" permission.
   */
  public function testEditAnyPermission() {
    $editor = $this->createUser(['translators_content edit any article content']);
    $this->drupalLogin($editor);
    $this->registerTestSkills();

    $nid = $this->createTestNode();
    $this->addTestTranslations(Node::load($nid));

    foreach (static::$registeredSkills as $skill) {
      if ($skill === $this->defaultLanguage) {
        $this->drupalGet("node/$nid/edit");
        $this->assertResponseCode(200);
      }
      else {
        $this->drupalGet("$skill/node/$nid/edit");
        $this->assertResponseCode(403);
      }
    }
    foreach (static::$unregisteredSkills as $skill) {
      $this->drupalGet("$skill/node/$nid/edit");
      $this->assertResponseCode(403);
      $this->assertTextHelper('Access denied', FALSE);
    }
  }

  /**
   * Test "Delete any content" permission.
   */
  public function testDeleteAnyPermission() {
    $deleter = $this->createUser(['translators_content delete any article content']);
    $this->drupalLogin($deleter);
    $this->registerTestSkills();

    $nid = $this->createTestNode();
    $this->addTestTranslations(Node::load($nid));

    foreach (static::$registeredSkills as $skill) {
      if ($skill === $this->defaultLanguage) {
        $this->drupalGet("node/$nid/delete");
        $this->assertResponseCode(200);
      }
      else {
        $this->drupalGet("$skill/node/$nid/delete");
        $this->assertResponseCode(403);
      }
    }
    foreach (static::$unregisteredSkills as $skill) {
      $this->drupalGet("$skill/node/$nid/delete");
      $this->assertResponseCode(403);
      $this->assertTextHelper('Access denied', FALSE);
    }
  }

  /**
   * Test case when user has permission to create and delete but not to edit.
   */
  public function testMissingEditPermissions() {
    $user = $this->createUser([
      'translators_content create article content',
      'translators_content delete any article content',
    ]);
    $this->drupalLogin($user);
    $this->registerTestSkills();

    // Ensure that user doesn't have edit permissions.
    $has_edit_permissions = $user->hasPermission('translators_content edit any article content');
    $this->assertFalse($has_edit_permissions);

    $nid = $this->createTestNode();
    $this->drupalGet("node/$nid/edit");
    $this->assertResponseCode(403);
    $this->assertTextHelper('Access denied', FALSE);
  }

  /**
   * Test language options on node create form with limited by skills option.
   */
  public function testLanguageOptionsInCreateForm() {
    $this->drupalLogin(
      $this->createUser([
        'translators_content create article content',
      ])
    );
    $this->registerTestSkills();

    $this->drupalGet('node/add/article');
    $this->assertResponseCode(200);

    // Find langcode field element.
    $langcode_field = $this->getSession()
      ->getPage()
      ->findField('langcode[0][value]');
    $this->assertNotNull($langcode_field);

    // Get all existing options of the langcode field dropdown.
    $options = $langcode_field->findAll('xpath', '//option');
    $this->assertNotNull($options);

    // Prepare array of options' values.
    $language_options = array_map(function ($option) {
      return $option->getAttribute('value') ?: $option->getText();
    }, $options);

    // Check that we still have "und" and "zxx" langcode options.
    foreach (static::$ignoredLangcodes as $langcode) {
      $this->assertContains($langcode, $language_options);
    }

    // Check that we have all registered skills.
    foreach (static::$registeredSkills as $langcode) {
      $this->assertContains($langcode, $language_options);
      if ($langcode !== 'en') {
        // Additionally check that the direct translation URL is accessible.
        $this->drupalGet($langcode . '/node/add/article');
        $this->assertResponseCode(200);
      }
    }

    // Check that we DO NOT have any unregistered skills options.
    foreach (static::$unregisteredSkills as $langcode) {
      $this->assertNotContains($langcode, $language_options);
      // Additionally check that the direct translation URL is NOT accessible.
      $this->drupalGet($langcode . '/node/add/article');
      $this->assertResponseCodeNot(200);
      $this->assertResponseCode(403);
    }
  }

  /**
   * Test that ensures we don't restrict access to the user's edit form.
   *
   * @throws \Behat\Mink\Exception\ExpectationException
   */
  public function testUserEditFormWorkaround() {
    $user = $this->createUser();
    $this->drupalLogin($user);

    // Check entity local task tabs existence.
    $this->drupalGet("user/{$user->id()}");
    $this->assertResponseCode(200);
    $this->assertSession()->elementExists('xpath', '//a[text()=\'Edit\']/@href');

    // Check for the edit page access.
    $this->drupalGet("user/{$user->id()}/edit");
    $this->assertResponseCodeNot(403);
    $this->assertResponseCode(200);

    // Additionally check that we don't give access to this form
    // for anonymous users.
    $this->drupalLogout();
    $this->drupalGet("user/{$user->id()}/edit");
    $this->assertResponseCode(403);
  }

}
