<?php

namespace Drupal\Tests\translators_content\Functional;

use Drupal\node\Entity\Node;
use Drupal\Tests\BrowserTestBase;
use Drupal\language\Entity\ConfigurableLanguage;

/**
 * Class TranslatorsContentTranslationSourceLanguagTest.
 *
 * @package Drupal\Tests\translators_content\Functional
 *
 * @group translators_content
 */
class TranslatorsContentTranslationSourceLanguagTest extends BrowserTestBase {
  use TranslatorsContentTestsTrait;

  /**
   * {@inheritdoc}
   */
  public $profile = 'standard';
  /**
   * {@inheritdoc}
   */
  public static $modules = ['translators_content'];

  /**
   * {@inheritdoc}
   */
  public function setUp() {
    parent::setUp();
    $this->setUpTest();
  }

  /**
   * Test access to the translation routes.
   */
  public function testRoutesAccess() {
    $nid = $this->createTestNode();
    Node::load($nid)
      ->addTranslation('de', ['title' => $this->randomString()])
      ->save();

    $creator = $this->createUser(
      ['translators_content create content translations', 'translate any entity'],
      'creator'
    );
    $this->drupalLogin($creator);
    $this->registerTestSkills();
    $this->drupalGet("fr/node/1/translations/add/en/fr");
    $this->assertResponseCode(200);
    $this->assertResponseCodeNot(403);

    $this->drupalGet("fr/node/1/translations/add/de/fr");
    $this->assertResponseCode(403);
    $this->assertResponseCodeNot(200);
  }

  /**
   * Test links existence.
   */
  public function testLinksExistence() {
    $nid = $this->createTestNode();
    Node::load($nid)
      ->addTranslation('de', ['title' => $this->randomString()])
      ->save();

    $creator = $this->createUser(
      ['translators_content create content translations', 'translate any entity'],
      'creator'
    );
    $this->drupalLogin($creator);
    $this->registerTestSkills();
    $this->drupalGet('node/1/translations');
    $this->assertResponseCode(200);
    $this->assertSession()
      ->elementExists('xpath', '//a[@hreflang=\'fr\'][text()=\'Add\']/@href');

    // Create node with German original language.
    $node = Node::create([
      'type'     => 'article',
      'title'    => $this->randomString(),
      'langcode' => 'de',
    ]);
    $node->save();

    $this->drupalGet('node/2/translations');
    $this->assertResponseCode(200);
    $this->assertSession()
      ->elementNotExists('xpath', '//a[@href=\'/fr/node/2/translations/add/de/fr\'][@hreflang=\'fr\'][text()=\'Add\']/@href');
  }

  /**
   * Test the non-default original language behavior.
   */
  public function testNonDefaultOriginalLanguage() {
    $node = Node::create([
      'type'     => 'article',
      'title'    => $this->randomString(),
      'langcode' => 'de',
    ]);
    $node->save();
    $creator = $this->createUser(
      ['translators_content create content translations', 'translate any entity'],
      'creator'
    );
    $this->drupalLogin($creator);
    $this->addSkill(static::$unregisteredSkills);

    $this->drupalGet('node/1/translations');
    $this->assertResponseCode(200);
    $this->assertSession()
      ->elementExists('xpath', '//a[@hreflang=\'sq\'][text()=\'Add\']/@href');
  }

  /**
   * Test Access By Strict Translation Skills Pairing.
   */
  public function testAccessByStrictTranslationSkillsPairing() {
    foreach (['am', 'nl'] as $language) {
      $this->assertEquals(1, ConfigurableLanguage::createFromLangcode($language)->save());
    }
    $this->drupalLogin($this->rootUser);
    // Create testing node.
    $this->drupalPostForm('node/add/article', [
      'title[0][value]' => $this->randomString(),
    ], 'Save');
    Node::load(1)
      ->addTranslation('am', ['title' => $this->randomString()])
      ->save();
    Node::load(1)
      ->addTranslation('de', ['title' => $this->randomString()])
      ->save();
    $this->drupalLogout();

    $translator = $this->createUser([
      'translators_content create content translations',
      'translate article node',
    ]);
    $this->drupalLogin($translator);
    $this->addSkill(static::$registeredSkills);
    $this->addSkill(['am', 'nl']);
    foreach (static::$registeredSkills + ['am', 'nl'] as $langcode) {
      $this->assertTrue($this->translatorSkills->hasLangcode($langcode));
    }

    $this->drupalGet('nl/node/1/translations/add/am/nl');
    $this->assertResponseCode(200);
    $this->drupalGet('nl/node/1/translations/add/am/sq');
    $this->assertResponseCode(403);
    // Check with and without strict translation paring option.
    $this->drupalGet('nl/node/1/translations/add/en/nl');
    $this->assertResponseCode(403);
    $this->drupalLogout();

    $this->drupalLogin($this->rootUser);
    $this->enableAllowAccessByStrictTranslationSkillsPairing(FALSE);
    $this->drupalLogout();

    $this->drupalLogin($translator);
    $this->drupalGet('nl/node/1/translations/add/en/nl');
    $this->assertResponseCode(200);

  }

  /**
   * Test source lang options on node create form with limited by skills option.
   */
  public function testSourceLanguageOptionsInCreateTranslationForm() {
    foreach (['am', 'nl'] as $language) {
      $this->assertEquals(1, ConfigurableLanguage::createFromLangcode($language)->save());
    }
    $this->drupalLogin($this->rootUser);
    // Create testing node.
    $this->drupalPostForm('node/add/article', [
      'title[0][value]' => $this->randomString(),
    ], 'Save');
    Node::load(1)
      ->addTranslation('am', ['title' => $this->randomString()])
      ->save();
    Node::load(1)
      ->addTranslation('fr', ['title' => $this->randomString()])
      ->save();
    Node::load(1)
      ->addTranslation('de', ['title' => $this->randomString()])
      ->save();
    $this->drupalLogout();

    $this->drupalLogin(
      $this->createUser([
        'translators_content create content translations',
        'translate article node',
      ])
    );
    $this->addSkill(static::$registeredSkills);
    $this->addSkill(['am', 'nl']);
    foreach (static::$registeredSkills + ['am', 'nl'] as $langcode) {
      $this->assertTrue($this->translatorSkills->hasLangcode($langcode));
    }

    $this->drupalGet('nl/node/1/translations/add/am/nl');
    $this->assertResponseCode(200);

    // Find langcode field element.
    $source_langcode_field = $this->getSession()
      ->getPage()
      ->findField('source_langcode[source]');
    $this->assertNotNull($source_langcode_field);

    // Get all existing options of the langcode field dropdown.
    $options = $source_langcode_field->findAll('xpath', '//option');
    $this->assertNotNull($options);

    // Prepare array of options' values.
    $language_options = array_map(function ($option) {
      return $option->getAttribute('value') ?: $option->getText();
    }, $options);

    $this->assertCount(1, $language_options);
    $this->assertContains('am', $language_options);

    $this->drupalLogin(
      $this->createUser([
        'create content translations',
        'translate article node',
      ])
    );
    $this->addSkill(static::$registeredSkills);
    $this->addSkill(['fr', 'sq']);
    foreach (static::$registeredSkills + ['fr', 'sq'] as $langcode) {
      $this->assertTrue($this->translatorSkills->hasLangcode($langcode));
    }

    $this->drupalGet('sq/node/1/translations/add/fr/sq');
    $this->assertResponseCode(200);

    // Find source langcode field element.
    $source_langcode_field = $this->getSession()
      ->getPage()
      ->findField('source_langcode[source]');
    $this->assertNotNull($source_langcode_field);

    $this->assertEquals('fr', $source_langcode_field->getValue());

    $skills_group = array_map(function ($option) {
      return $option->getAttribute('value') ?: $option->getText();
    }, $source_langcode_field->findAll('xpath', '//optgroup[@label="Translation skills"]/option'));
    $others_group = array_map(function ($option) {
      return $option->getAttribute('value') ?: $option->getText();
    }, $source_langcode_field->findAll('xpath', '//optgroup[@label="Others"]/option'));

    $this->assertNotEmpty($skills_group);
    $this->assertCount(2, $skills_group);
    $this->assertContains('fr', $skills_group);
    $this->assertContains('en', $skills_group);

    $this->assertNotEmpty($others_group);
    $this->assertCount(2, $others_group);
    $this->assertContains('am', $others_group);
    $this->assertContains('de', $others_group);
  }

}
