<?php

namespace Drupal\Tests\translators_content\Functional;

use Drupal\Tests\BrowserTestBase;
use Drupal\Tests\node\Traits\NodeCreationTrait;
use Drupal\Core\Url;
use Drupal\user\UserInterface;

/**
 * Class TranslatorsContentTranslationWorkflowsTest.
 *
 * @package Drupal\Tests\translators_content\Functional
 *
 * @group translators_content
 */
class TranslatorsContentTranslationWorkflowsTest extends BrowserTestBase {
  use TranslatorsContentTestsTrait, NodeCreationTrait;

  /**
   * {@inheritdoc}
   */
  public $profile = 'standard';

  /**
   * {@inheritdoc}
   */
  public static $modules = ['translators_content'];

  /**
   * Node to test.
   *
   * @var object
   */
  protected $node;

  /**
   * The added languages.
   *
   * @var array
   */
  protected $langcodes;

  /**
   * {@inheritdoc}
   */
  public function setUp() {
    parent::setUp();
    $this->langcodes = $this->getAllTestingLanguages();
    // Make all required configurations before testing.
    $this->setUpTest();
  }

  /**
   * User has no permissions for translation except 'translate any entity'.
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function testNonePermissions() {
    $user = $this->createUser(['translate any entity']);

    // Test workflows for the user with no permissions.
    $expected_status = [
      'edit' => 403,
      'delete' => 403,
      'translation_overview' => 403,
      'translation_add' => 403,
      'translation_edit' => 403,
      'translation_delete' => 403,
    ];
    $this->doTestWorkflows($user, $expected_status);
  }

  /**
   * Test case for editor.
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function testEditorPermissions() {
    $user = $this->createUser([
      'edit any article content',
      'delete any article content',
    ]);

    // Test workflows for the editor.
    $expected_status = [
      'edit' => 200,
      'delete' => 200,
      'translation_overview' => 403,
      'translation_add' => 403,
      'translation_edit' => 403,
      'translation_delete' => 403,
    ];
    $this->doTestWorkflows($user, $expected_status);
  }

  /**
   * Test case for 'bypass node access' permission.
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function testBypassNodeAccessPermissions() {
    $user = $this->createUser(['bypass node access']);

    // Test workflows for the editor.
    $expected_status = [
      'edit' => 200,
      'delete' => 200,
      'translation_overview' => 200,
      'translation_add' => 200,
      'translation_edit' => 200,
      'translation_delete' => 200,
    ];
    $this->doTestWorkflows($user, $expected_status);
  }

  /**
   * Test case for translator.
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function testTranslatorPermissions() {
    $user = $this->createUser([
      'create content translations',
      'update content translations',
      'delete content translations',
      'translate any entity',
    ]);

    // Test workflows for the translator.
    $expected_status = [
      'edit' => 403,
      'delete' => 403,
      'translation_overview' => 200,
      'translation_add' => 200,
      'translation_edit' => 200,
      'translation_delete' => 200,
    ];
    $this->doTestWorkflows($user, $expected_status);
  }

  /**
   * Test case for admin.
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function testAdminPermissions() {
    $user = $this->createUser([]);
    $user->addRole('administrator');
    $user->save();

    // Test workflows for the admin.
    $expected_status = [
      'edit' => 200,
      'delete' => 200,
      'translation_overview' => 200,
      'translation_add' => 200,
      'translation_edit' => 200,
      'translation_delete' => 200,
    ];
    $this->doTestWorkflows($user, $expected_status);
  }

  /**
   * Additional steps for tests set up.
   */
  protected function setUpTest() {
    $this->drupalLogin($this->rootUser);
    $this->createLanguages();
    $this->enableTranslation('node', 'article');
    $this->enableAllowAccessByStrictTranslationSkillsPairing(FALSE);
    $this->enableAutoPresetSourceLanguage();
    $this->enableFilterTranslationOverviewToSkills(FALSE);
    $this->createTestArticle();
    $this->drupalLogout();
  }

  /**
   * Create test article with default language.
   */
  protected function createTestArticle() {
    $node = $this
      ->drupalCreateNode([
        'status' => TRUE,
        'type' => 'article',
        'langcode' => $this->defaultLanguage,
      ]);
    $node->save();

    $this->node = $node;

    // Create a translation.
    $translation_add_url = Url::fromRoute(
      "entity.node.content_translation_add",
      [
        'node' => $this->node->id(),
        'source' => $this->defaultLanguage,
        'target' => $this->langcodes[1],
      ]
    );
    $this->drupalPostForm($translation_add_url, [], t('Save'));
    $this->rebuildContainer();
  }

  /**
   * Checks that workflows have the expected behaviors for the given user.
   *
   * @param \Drupal\user\UserInterface $user
   *   The user to test the workflow behavior against.
   * @param array $expected_status
   *   The an associative array with the operation name as key and the expected
   *   status as value.
   */
  protected function doTestWorkflows(UserInterface $user, array $expected_status) {
    $default_langcode = $this->defaultLanguage;
    $languages = $this->container->get('language_manager')->getLanguages();
    $args = ['@user_label' => $user->getAccountName()];
    $options = ['language' => $languages[$default_langcode], 'absolute' => TRUE];
    $this->drupalLogin($user);
    // Check whether the user is allowed to access the entity form in edit mode.
    $edit_url = $this->node->toUrl('edit-form', $options);
    $this->drupalGet($edit_url, $options);
    $this->assertResponse($expected_status['edit']);

    // Check whether the user is allowed to access the entity delete form.
    $delete_url = $this->node->toUrl('delete-form', $options);
    $this->drupalGet($delete_url, $options);
    $this->assertResponse($expected_status['delete']);

    // Check whether the user is allowed to access the translation overview.
    $translation_overview_url = $this->node->toUrl('drupal:content-translation-overview', $options)->toString();
    $this->drupalGet($translation_overview_url);
    $this->assertResponse($expected_status['translation_overview']);

    // Check whether the user is allowed to create a translation.
    $langcode = $this->langcodes[2];
    $options['language'] = $languages[$langcode];
    $translation_add_url = Url::fromRoute(
      "entity.node.content_translation_add",
      [
        'node' => $this->node->id(),
        'source' => $default_langcode,
        'target' => $langcode,
      ],
      $options
    );
    if ($expected_status['translation_add'] == 200) {
      $this->click('.add a[hreflang="' . $langcode . '"]');
      $this->assertUrl($translation_add_url->toString());
    }
    else {
      $this->drupalGet($translation_add_url);
    }
    $this->assertResponse($expected_status['translation_add']);

    // Check whether the user is allowed to edit a translation.
    $langcode = $this->langcodes[1];
    $options['language'] = $languages[$langcode];
    $translation_edit_url = Url::fromRoute(
      "entity.node.content_translation_edit",
      [
        'node' => $this->node->id(),
        'language' => $langcode,
      ],
      $options
    );
    if ($expected_status['translation_edit'] == 200) {
      $this->drupalGet($translation_overview_url);
      $this->assertResponse($expected_status['translation_overview']);
      $this->assertSession()
        ->elementExists('css', '.edit a[hreflang="' . $langcode . '"]');
      $this->click('.edit a[hreflang="' . $langcode . '"]');
    }
    else {
      $this->drupalGet($translation_edit_url);
    }
    $this->assertResponse($expected_status['translation_edit']);

    // Check whether the user is allowed to delete a translation.
    $translation_delete_url = Url::fromRoute(
      "entity.node.content_translation_delete",
      [
        'node' => $this->node->id(),
        'language' => $langcode,
      ]
    );
    if ($expected_status['translation_delete'] == 200) {
      $this->drupalGet($translation_overview_url);
      $this->assertResponse($expected_status['translation_overview']);
      $this->assertSession()
        ->elementExists('css', '.delete a[hreflang="' . $langcode . '"]');
      $this->click('.delete a[hreflang="' . $langcode . '"]');
    }
    else {
      $this->drupalGet($translation_delete_url);
    }
    $this->assertResponse($expected_status['translation_delete']);
  }

}
